#ifndef __CAMERA_H
#define __CAMERA_H

#include "Vector3.h"
#include "AxisAlignedBox3.h"
#include "Plane3.h"


namespace CHCDemoEngine 
{


class Camera
{
public:
	
	/** Small struct representing a frustum.
	*/
	struct Frustum
	{
		/// the 6 clip planes
		Plane3 mClipPlanes[6];

		void CalcNPVertexIndices(int *indices);
	};
	
	Camera();

	Camera(int width, int height, float fieldOfView = 90.f);


	void SetPosition(const Vector3 &pos);

	inline Vector3 GetPosition() const { return mPosition; }
	Vector3 GetDirection() const;
	Vector3 GetUpVector() const;
	Vector3 GetRightVector() const;

	inline float GetFov() const { return mFovy; }
	inline void GetSize(int &width, int &height) const { width = mWidth; height = mHeight; }
	inline float GetAspect() const { return (float) mWidth / mHeight; }

	/** Sets up viewing in gl
	*/
	void SetupCameraView();
	void GetProjectionMatrix(Matrix4x4 &mat);
	void GetModelViewMatrix(Matrix4x4 &mat);

	void CalcFrustum(Frustum &frustum);

	void ComputePoints(Vector3 &ftl, Vector3 &ftr, Vector3 &fbl, Vector3 &fbr,
		               Vector3 &ntl, Vector3 &ntr, Vector3 &nbl, Vector3 &nbr);
	
	inline float GetNear() const { return mNear; }
	void SetNear(float nearDist);
	
	inline float GetFar() const { return mFar; }

	void SetFar(float farDist) { mFar = farDist; }

	void SetOrtho(bool ortho);

	void Yaw(float angle);
	void Pitch(float angle);
	


protected:

	void Precompute(const Vector3 &direction);

	void CalculateFromPitchAndYaw();


	////////////////
	//-- members

	float mFovy;
	int mWidth;
	int mHeight;

	float mNear;

	float mFar;

	bool mIsOrtho;

	Matrix4x4 mBaseOrientation;
	Matrix4x4 mViewOrientation;

	float mPitch;
	float mYaw;

	Vector3 mPosition;
};

}
#endif
